# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: list_ledger_entries.py
DESCRIPTION:
    This sample demonstrates how to iteratively retrieve a batch of ledger entries. In this sample,
    we write many ledger entries before retrieving them at once.
USAGE:
    python list_ledger_entries.py
    Set the environment variables with your own values before running the sample:
    1) CONFIDENTIALLEDGER_ENDPOINT - the endpoint of the Confidential Ledger.
"""

import logging
import os
import sys
import tempfile

from azure.confidentialledger import ConfidentialLedgerClient
from azure.confidentialledger.certificate import (
    ConfidentialLedgerCertificateClient,
)
from azure.identity import DefaultAzureCredential


logging.basicConfig(level=logging.ERROR)
LOG = logging.getLogger()


def main():
    # Set the values of the client ID, tenant ID, and client secret of the AAD application as
    # environment variables:
    #   AZURE_CLIENT_ID, AZURE_TENANT_ID, AZURE_CLIENT_SECRET, CONFIDENTIALLEDGER_ENDPOINT
    try:
        ledger_endpoint = os.environ["CONFIDENTIALLEDGER_ENDPOINT"]
    except KeyError:
        LOG.error(
            "Missing environment variable 'CONFIDENTIALLEDGER_ENDPOINT' - "
            "please set it before running the example"
        )
        sys.exit(1)

    # Under the current URI format, the ledger id is the first part of the ledger endpoint.
    # i.e. https://<ledger id>.confidential-ledger.azure.com
    ledger_id = ledger_endpoint.replace("https://", "").split(".")[0]

    identity_service_client = ConfidentialLedgerCertificateClient()  # type: ignore[call-arg]
    ledger_certificate = identity_service_client.get_ledger_identity(ledger_id)

    # The Confidential Ledger's TLS certificate must be written to a file to be used by the
    # ConfidentialLedgerClient. Here, we write it to a temporary file so that is is cleaned up
    # automatically when the program exits.
    with tempfile.TemporaryDirectory() as tempdir:
        ledger_cert_file = os.path.join(tempdir, f"{ledger_id}.pem")
        with open(ledger_cert_file, "w") as outfile:
            outfile.write(ledger_certificate["ledgerTlsCertificate"])

        print(
            f"Ledger certificate has been written to {ledger_cert_file}. "
            "It will be deleted when the script completes."
        )

        # Build a client through AAD
        ledger_client = ConfidentialLedgerClient(
            ledger_endpoint,
            credential=DefaultAzureCredential(),
            ledger_certificate_path=ledger_cert_file,
        )

        post_poller = ledger_client.begin_create_ledger_entry({"contents": "First message"})  # type: ignore[attr-defined]
        first_transaction_id = post_poller.result()["transactionId"]

        print(
            "Wrote 'First message' to the ledger. It is recorded at transaction id "
            f"{first_transaction_id}."
        )

        for i in range(10):
            entry_contents = f"Message {i}"
            print(
                f"Writing '{entry_contents}' to the ledger."
            )

            ledger_client.create_ledger_entry({"contents": entry_contents})

        post_poller = ledger_client.begin_create_ledger_entry({"contents": "Last message"})  # type: ignore[attr-defined]
        last_transaction_id = post_poller.result()["transactionId"]

        print(
            "Wrote 'Last message' to the ledger. It is recorded at transaction id "
            f"{last_transaction_id}."
        )

        ranged_result = ledger_client.list_ledger_entries(
            from_transaction_id=first_transaction_id,
            to_transaction_id=last_transaction_id,
        )
        for entry in ranged_result:
            print(f'Contents at {entry["transactionId"]}: {entry["contents"]}')


if __name__ == "__main__":
    main()



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------


"""Customize generated code here.

Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
from typing import Any, List, Optional

from azure.confidentialledger.certificate._client import (
    ConfidentialLedgerCertificateClient as GeneratedClient,
)

__all__: List[str] = [
    "ConfidentialLedgerCertificateClient"
]  # Add all objects you want publicly available to users at this package level


def patch_sdk():
    """Do not remove from this file.

    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """


class ConfidentialLedgerCertificateClient(GeneratedClient):
    def __init__(  # pylint: disable=missing-client-constructor-parameter-credential
        self, certificate_endpoint: Optional[str] = None, **kwargs: Any
    ) -> None:
        """
        :param certificate_endpoint: The Identity Service URL, for example
         https://identity.confidential-ledger.core.azure.com, defaults to None. If not provided,
         "https://identity.confidential-ledger.core.azure.com" will be used.
        :type certificate_endpoint: Optional[str], optional
        :keyword api_version: Api Version. Default value is "2022-05-13". Note that overriding this
         default value may result in unsupported behavior.
        :paramtype api_version: str
        """

        if not certificate_endpoint:
            certificate_endpoint = "https://identity.confidential-ledger.core.azure.com"
        super().__init__(certificate_endpoint, **kwargs)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------


"""Customize generated code here.

Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
import os
from typing import Any, List, Union

from azure.core.credentials import TokenCredential
from azure.core.pipeline import policies

from azure.confidentialledger._client import ConfidentialLedgerClient as GeneratedClient
from azure.confidentialledger.certificate import ConfidentialLedgerCertificateClient

__all__: List[str] = [
    "ConfidentialLedgerCertificateCredential",
    "ConfidentialLedgerClient",
]  # Add all objects you want publicly available to users at this package level


def patch_sdk():
    """Do not remove from this file.

    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """


class ConfidentialLedgerCertificateCredential:
    """A certificate-based credential for the ConfidentialLedgerClient.

    :param certificate_path: Path to the PEM certificate to use for authentication.
    :type certificate_path: Union[bytes, str, os.PathLike]
    """

    def __init__(self, certificate_path: Union[bytes, str, os.PathLike]):
        self._certificate_path = certificate_path

    @property
    def certificate_path(self) -> Union[bytes, str, os.PathLike]:
        """The path to the certificate file for this credential.

        :return: The path to the certificate file for this credential.
        :rtype: Union[bytes, str, os.PathLike]"""

        return self._certificate_path


class ConfidentialLedgerClient(GeneratedClient):
    """The ConfidentialLedgerClient writes and retrieves ledger entries against the Confidential
    Ledger service.

    :param endpoint: The Confidential Ledger URL, for example
     https://contoso.confidentialledger.azure.com.
    :type endpoint: str
    :param credential: A credential object for authenticating with the Confidential Ledger.
    :type credential: Union[
        ~azure.confidentialledger.ConfidentialLedgerCertificateCredential,
        ~azure.core.credentials.TokenCredential]
    :keyword ledger_certificate_path: The path to the Confidential Ledger's TLS certificate. If this
        file does not exist yet, the Confidential Ledger's TLS certificate will be fetched and saved
        to this file.
    :paramtype ledger_certificate_path: Union[bytes, str, os.PathLike]
    :keyword api_version: Api Version. Default value is "2022-05-13". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    """

    def __init__(
        self,
        endpoint: str,
        credential: Union[ConfidentialLedgerCertificateCredential, TokenCredential],
        *,
        ledger_certificate_path: Union[bytes, str, os.PathLike],
        **kwargs: Any,
    ) -> None:
        # Remove some kwargs first so that there aren't unexpected kwargs passed to
        # get_ledger_identity.

        if isinstance(credential, ConfidentialLedgerCertificateCredential):
            auth_policy = None
        else:
            credential_scopes = kwargs.pop("credential_scopes", ["https://confidential-ledger.azure.com/.default"])
            auth_policy = kwargs.pop(
                "authentication_policy",
                policies.BearerTokenCredentialPolicy(credential, *credential_scopes, **kwargs),
            )
        if os.path.isfile(ledger_certificate_path) is False:
            # We'll need to fetch the TLS certificate.

            identity_service_client = ConfidentialLedgerCertificateClient(**kwargs)

            # Ledger URIs are of the form https://<ledger id>.confidential-ledger.azure.com.

            ledger_id = endpoint.replace("https://", "").split(".")[0]
            ledger_cert = identity_service_client.get_ledger_identity(ledger_id, **kwargs)

            with open(ledger_certificate_path, "w", encoding="utf-8") as outfile:
                outfile.write(ledger_cert["ledgerTlsCertificate"])
        # For ConfidentialLedgerCertificateCredential, pass the path to the certificate down to the
        # PipelineCLient.

        if isinstance(credential, ConfidentialLedgerCertificateCredential):
            kwargs["connection_cert"] = kwargs.get("connection_cert", credential.certificate_path)
        # The auto-generated client has authentication disabled so we can customize authentication.
        # If the credential is the typical TokenCredential, then construct the authentication policy
        # the normal way.

        else:
            kwargs["authentication_policy"] = auth_policy
        # Customize the underlying client to use a self-signed TLS certificate.

        kwargs["connection_verify"] = kwargs.get("connection_verify", ledger_certificate_path)

        super().__init__(endpoint, **kwargs)



from devtools_testutils import AzureRecordedTestCase, recorded_by_proxy

from azure.confidentialledger.certificate import (
    ConfidentialLedgerCertificateClient,
)

from _shared.testcase import ConfidentialLedgerPreparer


class TestConfidentialLedgerCertificateClient(AzureRecordedTestCase):
    @ConfidentialLedgerPreparer()
    @recorded_by_proxy
    def test_get_ledger_identity(self, **kwargs):
        confidentialledger_id = kwargs.pop("confidentialledger_id")

        client = self.create_client_from_credential(
            ConfidentialLedgerCertificateClient,
            credential=None,
        )

        network_identity = (
            client.get_ledger_identity(
                ledger_id=confidentialledger_id
            )
        )

        assert network_identity["ledgerId"] == confidentialledger_id
        assert network_identity["ledgerTlsCertificate"]



from devtools_testutils import AzureRecordedTestCase
from devtools_testutils.aio import recorded_by_proxy_async

from azure.confidentialledger.certificate.aio import (
    ConfidentialLedgerCertificateClient,
)

from _shared.testcase import ConfidentialLedgerPreparer


class TestConfidentialLedgerCertificateClient(AzureRecordedTestCase):
    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_get_ledger_identity(self, **kwargs):
        confidentialledger_id = kwargs.pop("confidentialledger_id")

        client = self.create_client_from_credential(
            ConfidentialLedgerCertificateClient,
            credential=None,
        )

        try:
            network_identity = (
                await client.get_ledger_identity(
                    ledger_id=confidentialledger_id
                )
            )

            assert network_identity["ledgerId"] == confidentialledger_id
            assert network_identity["ledgerTlsCertificate"]
        finally:
            await client.close()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: get_and_verify_receipt.py
DESCRIPTION:
    This sample demonstrates how to retrieve Confidential Ledger receipts
    and verify their content. In this sample, we write a ledger entry, retrieve
    a receipt certifying that it was written correctly, and then verify its
    content by applying the receipt verification algorithm.
USAGE:
    python get_and_verify_receipt.py
    Set the environment variables with your own values before running the sample:
    1) CONFIDENTIALLEDGER_ENDPOINT - the endpoint of the Confidential Ledger.
"""

import logging
import os
import sys
import tempfile

from azure.confidentialledger import ConfidentialLedgerClient
from azure.confidentialledger.certificate import (
    ConfidentialLedgerCertificateClient,
)
from azure.confidentialledger.receipt import (
    verify_receipt,
)
from azure.core.exceptions import HttpResponseError
from azure.identity import DefaultAzureCredential


logging.basicConfig(level=logging.ERROR)
LOG = logging.getLogger()


def main():
    """Main method."""

    # Set the values of the client ID, tenant ID, and client secret of the AAD application as
    # environment variables:
    #   AZURE_CLIENT_ID, AZURE_TENANT_ID, AZURE_CLIENT_SECRET, CONFIDENTIALLEDGER_ENDPOINT
    try:
        ledger_endpoint = os.environ["CONFIDENTIALLEDGER_ENDPOINT"]
    except KeyError:
        LOG.error(
            "Missing environment variable 'CONFIDENTIALLEDGER_ENDPOINT' - "
            "please set it before running the example"
        )
        sys.exit(1)

    # Under the current URI format, the ledger id is the first part of the ledger endpoint.
    # i.e. https://<ledger id>.confidential-ledger.azure.com
    ledger_id = ledger_endpoint.replace("https://", "").split(".")[0]

    identity_service_client = ConfidentialLedgerCertificateClient()  # type: ignore[call-arg]
    ledger_certificate = identity_service_client.get_ledger_identity(ledger_id)

    # The Confidential Ledger's TLS certificate must be written to a file to be used by the
    # ConfidentialLedgerClient. Here, we write it to a temporary file so that is is cleaned up
    # automatically when the program exits.
    with tempfile.TemporaryDirectory() as tempdir:
        ledger_cert_file = os.path.join(tempdir, f"{ledger_id}.pem")
        with open(ledger_cert_file, "w") as outfile:
            outfile.write(ledger_certificate["ledgerTlsCertificate"])

        print(
            f"Ledger certificate has been written to {ledger_cert_file}. "
            "It will be deleted when the script completes."
        )

        # Build a client through AAD
        ledger_client = ConfidentialLedgerClient(
            ledger_endpoint,
            credential=DefaultAzureCredential(),
            ledger_certificate_path=ledger_cert_file,
        )

        # Write a ledger entry and wait for the transaction to be committed.
        entry_contents = "Hello world!"
        post_poller = ledger_client.begin_create_ledger_entry(
            {"contents": entry_contents}
        )
        post_entry_result = post_poller.result()
        transaction_id = post_entry_result["transactionId"]
        print(
            f"Wrote '{entry_contents}' to the ledger at transaction {transaction_id}."
        )

        # Get a receipt for a ledger entry.
        # A receipt can be retrieved for any transaction id to provide cryptographic proof of the
        # contents of the transaction.
        print(
            f"Retrieving a receipt for {transaction_id}. The receipt may be used to "
            "cryptographically verify the contents of the transaction."
        )
        print(
            "For more information about receipts, please see "
            "https://microsoft.github.io/CCF/main/audit/receipts.html#receipts"
        )
        get_receipt_poller = ledger_client.begin_get_receipt(transaction_id)
        get_receipt_result = get_receipt_poller.result()
        print(f"Receipt for transaction id {transaction_id}: {get_receipt_result}")

        # Read content of service certificate file saved in previous step.
        with open(ledger_cert_file, "r") as service_cert_file:
            service_cert_content = service_cert_file.read()

        # Optionally read application claims, if any
        application_claims = get_receipt_result.get("applicationClaims", None)

        try:
            # Verify the contents of the receipt.
            verify_receipt(
                get_receipt_result["receipt"],
                service_cert_content,
                application_claims=application_claims,
            )
            print(f"Receipt for transaction id {transaction_id} successfully verified")
        except ValueError:
            print(f"Receipt verification for transaction id {transaction_id} failed")
            raise


if __name__ == "__main__":
    main()



import functools
import os
import tempfile

from devtools_testutils import (
    AzureRecordedTestCase,
    EnvironmentVariableLoader,
)

from azure.confidentialledger.certificate import (
    ConfidentialLedgerCertificateClient,
)
from azure.confidentialledger.certificate.aio import (
    ConfidentialLedgerCertificateClient as ConfidentialLedgerCertificateClientAsync
)

from .constants import USER_CERTIFICATE


ConfidentialLedgerPreparer = functools.partial(
    EnvironmentVariableLoader,
    "confidentialledger",
    confidentialledger_id="fake",
    confidentialledger_endpoint="https://fake.confidential-ledger.azure.com",
    confidentialledger_resource_group="fakegroup",
)


class ConfidentialLedgerTestCase(AzureRecordedTestCase):
    @classmethod
    def setup_class(cls):
        """setup any state specific to the execution of the given class (which
        usually contains tests).
        """

        with tempfile.NamedTemporaryFile(
            "w", suffix=".pem", delete=False
        ) as tls_cert_file:
            cls.network_certificate_path = tls_cert_file.name

        with tempfile.NamedTemporaryFile(
            "w", suffix=".pem", delete=False
        ) as user_cert_file:
            user_cert_file.write(USER_CERTIFICATE)
            cls.user_certificate_path = user_cert_file.name

    @classmethod
    def teardown_class(cls):
        """teardown any state that was previously setup with a call to
        setup_class.
        """
        try:
            os.remove(cls.user_certificate_path)
        except FileNotFoundError:
            pass

        if cls.network_certificate_path:
            try:
                os.remove(cls.network_certificate_path)
            except FileNotFoundError:
                pass

    def set_ledger_identity(self, confidentialledger_id: str) -> str:
        """Retrieves the Confidential Ledger's TLS certificate, saving it to the object's network
        certificate path as well as returning it directly.

        :param confidentialledger_id: Id of the Confidential Ledger.
        :type confidentialledger_id: str
        :return: The Confidential Ledger's TLS certificate.
        :rtype: str
        """
        client = self.create_client_from_credential(
            ConfidentialLedgerCertificateClient,
            credential=None,
        )

        network_identity = (
            client.get_ledger_identity(
                ledger_id=confidentialledger_id
            )
        )

        with open(self.network_certificate_path, "w", encoding="utf-8") as outfile:
            outfile.write(network_identity["ledgerTlsCertificate"])

        return network_identity["ledgerTlsCertificate"]

    async def set_ledger_identity_async(self, confidentialledger_id: str) -> str:
        """Retrieves the Confidential Ledger's TLS certificate, saving it to the object's network
        certificate path as well as returning it directly.

        An async version of this method is needed so that this request is recorded by async tests.

        :param confidentialledger_id: Id of the Confidential Ledger.
        :type confidentialledger_id: str
        :return: The Confidential Ledger's TLS certificate.
        :rtype: str
        """
        client = self.create_client_from_credential(
            ConfidentialLedgerCertificateClientAsync,
            credential=None,
        )

        try:
            network_identity = (
                await client.get_ledger_identity(
                    ledger_id=confidentialledger_id
                )
            )

            with open(self.network_certificate_path, "w", encoding="utf-8") as outfile:
                outfile.write(network_identity["ledgerTlsCertificate"])

            return network_identity["ledgerTlsCertificate"]
        finally:
            await client.close()



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""Tests for receipt verification."""

import pytest

from azure.confidentialledger.receipt._receipt_verification import (
    verify_receipt,
)

from azure.confidentialledger.receipt._receipt_models import (
    LeafComponents,
    ProofElement,
)

from ._shared.receipt_constants import (
    get_test_valid_receipt_1_dict,
    get_test_valid_receipt_2_dict,
    get_test_valid_service_certificate_2,
    get_test_valid_service_certificate_1,
)

from ._shared.claims_constants import (
    get_test_application_claims_with_claim_digest_dict,
    get_test_application_claims_with_ledger_entry_dict,
)


@pytest.mark.parametrize(
    "input_receipt,input_service_cert",
    [
        [get_test_valid_receipt_1_dict(), get_test_valid_service_certificate_1()],
        [get_test_valid_receipt_2_dict(), get_test_valid_service_certificate_2()],
    ],
)
def test_receipt_verification_with_valid_receipt_returns_successfully(
    input_receipt, input_service_cert
):
    # Check that verify_receipt does not throw any exception
    # with a valid receipt and service certificate
    try:
        verify_receipt(input_receipt, input_service_cert)
    except Exception as e:
        pytest.fail(
            f"verify_receipt threw an exception with a valid receipt and service certificate {e}"
        )


@pytest.mark.parametrize(
    "input_leaf_components",
    [
        None,
        LeafComponents(
            claimsDigest="invalid_claims_digest",
            commitEvidence="invalid_commit_evidence",
            writeSetDigest="invalid_write_set_digest",
        ),
        LeafComponents(
            claimsDigest=get_test_valid_receipt_1_dict()["leaf_components"][
                "claims_digest"
            ],
            commitEvidence="invalid_commit_evidence",
            writeSetDigest="invalid_write_set_digest",
        ),
        LeafComponents(
            claimsDigest="invalid_claims_digest",
            commitEvidence="invalid_commit_evidence",
            writeSetDigest=get_test_valid_receipt_1_dict()["leaf_components"][
                "write_set_digest"
            ],
        ),
    ],
)
def test_receipt_verification_with_invalid_leaf_components_throws_exception(
    input_leaf_components,
):
    # Create a receipt with invalid leaf components
    receipt = get_test_valid_receipt_1_dict()
    receipt["leafComponents"] = input_leaf_components

    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


@pytest.mark.parametrize("input_node_cert", [None, "invalid_cert_string"])
def test_receipt_verification_with_invalid_node_cert_throws_exception(input_node_cert):
    # Create a receipt with an invalid node certificate
    receipt = get_test_valid_receipt_1_dict()
    receipt["cert"] = input_node_cert

    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


@pytest.mark.parametrize(
    "input_node_id",
    [None, "invalid_node_id", get_test_valid_receipt_2_dict()["nodeId"]],
)
def test_receipt_verification_with_invalid_node_id_throws_exception(input_node_id):
    # Create a receipt with an invalid node id
    receipt = get_test_valid_receipt_1_dict()
    receipt["node_id"] = input_node_id

    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


def test_receipt_verification_with_missing_node_id_does_not_throw_exception():
    # Create a receipt with an invalid node id
    receipt = get_test_valid_receipt_1_dict()
    del receipt["node_id"]

    # Check that verify_receipt does throw any exception
    try:
        verify_receipt(receipt, get_test_valid_service_certificate_1())
    except Exception as e:
        pytest.fail(
            f"verify_receipt threw an exception with a valid receipt and service certificate {e}"
        )


@pytest.mark.parametrize(
    "input_proof_list",
    [
        None,
        [ProofElement()],
        [ProofElement(right="invalid_node_hash")],
        [ProofElement(right="invalid_node_hash")],
        [ProofElement(left="invalid_node_hash", right="invalid_node_hash")],
    ],
)
def test_receipt_verification_with_invalid_proof_list_throws_exception(
    input_proof_list,
):
    # Create a receipt with an invalid proof list
    receipt = get_test_valid_receipt_1_dict()
    receipt["proof"] = input_proof_list

    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


@pytest.mark.parametrize("input_endorsements_list", [1, 2])
def test_receipt_verification_with_invalid_service_endorsements_throws_exception(
    input_endorsements_list,
):
    # Create a receipt with a null endorsements list
    receipt = get_test_valid_receipt_1_dict()
    receipt["service_endorsements"] = input_endorsements_list

    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


def test_receipt_verification_with_missing_service_endorsements_does_not_throw_exception():
    # Create a receipt and remove the endorsements list
    receipt = get_test_valid_receipt_1_dict()
    del receipt["service_endorsements"]

    # Check that verify_receipt does throw any exception
    try:
        verify_receipt(receipt, get_test_valid_service_certificate_1())
    except Exception as e:
        pytest.fail(
            f"verify_receipt threw an exception with a valid receipt and service certificate {e}"
        )


@pytest.mark.parametrize(
    "input_signature", [None, get_test_valid_receipt_2_dict()["signature"]]
)
def test_receipt_verification_with_invalid_signature_throws_exception(input_signature):
    # Create a receipt with an invalid signature
    receipt = get_test_valid_receipt_1_dict()
    receipt["signature"] = input_signature

    # Check that verify_receipt throws ValueError
    with pytest.raises(ValueError):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


@pytest.mark.parametrize("input_service_cert", [None, "invalid_cert_string"])
def test_receipt_verification_with_invalid_service_cert_throws_exception(
    input_service_cert,
):
    # Check that verify_receipt throws ValueError with invalid service certificate
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(get_test_valid_receipt_1_dict(), input_service_cert)


@pytest.mark.parametrize("input_service_cert", [get_test_valid_service_certificate_2()])
def test_receipt_verification_with_unknown_service_cert_throws_exception(
    input_service_cert,
):
    # Check that verify_receipt throws ValueError with unknown service certificate
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(get_test_valid_receipt_1_dict(), input_service_cert)


@pytest.mark.parametrize("input_node_cert", [get_test_valid_receipt_2_dict()["cert"]])
def test_receipt_verification_with_unknown_node_cert_throws_exception(input_node_cert):
    # Create a receipt with unknown node certificate
    receipt = get_test_valid_receipt_1_dict()
    receipt["cert"] = input_node_cert

    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


@pytest.mark.parametrize(
    "input_endorsements_list", [[get_test_valid_service_certificate_2()]]
)
def test_receipt_verification_with_unknown_service_endorsements_throws_exception(
    input_endorsements_list,
):
    # Create a receipt with unknown endorsements list
    receipt = get_test_valid_receipt_1_dict()
    receipt["service_endorsements"] = input_endorsements_list

    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


@pytest.mark.parametrize(
    "input_leaf_components", [get_test_valid_receipt_2_dict()["leafComponents"]]
)
def test_receipt_verification_with_unknown_leaf_components_throws_exception(
    input_leaf_components,
):
    # Create a receipt with unknown leaf components
    receipt = get_test_valid_receipt_1_dict()
    receipt["leafComponents"] = input_leaf_components

    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


@pytest.mark.parametrize(
    "input_proof_list", [[], get_test_valid_receipt_2_dict()["proof"]]
)
def test_receipt_verification_with_unknown_proof_list_throws_exception(
    input_proof_list,
):
    # Create a receipt with unknown proof list
    receipt = get_test_valid_receipt_1_dict()
    receipt["proof"] = input_proof_list

    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(receipt, get_test_valid_service_certificate_1())


@pytest.mark.parametrize(
    "input_receipt,input_service_cert,input_claims",
    [
        [
            {
                "cert": "-----BEGIN CERTIFICATE-----\nMIIBxTCCAUygAwIBAgIRAMR89lUNeIghDUfpyHi3QzIwCgYIKoZIzj0EAwMwFjEU\nMBIGA1UEAwwLQ0NGIE5ldHdvcmswHhcNMjMwNDI1MTgxNDE5WhcNMjMwNzI0MTgx\nNDE4WjATMREwDwYDVQQDDAhDQ0YgTm9kZTB2MBAGByqGSM49AgEGBSuBBAAiA2IA\nBB1DiBUBr9/qapmvAIPm1o3o3LRViSOkfFVI4oPrw3SodLlousHrLz+HIe+BqHoj\n4nBjt0KAS2C0Av6Q+Xg5Po6GCu99GQSoSfajGqmjy3j3bwjsGJi5wHh1pNbPmMm/\nTqNhMF8wDAYDVR0TAQH/BAIwADAdBgNVHQ4EFgQUCPaDohOGjVgQ2Lb8Pmubg7Y5\nDJAwHwYDVR0jBBgwFoAU25KejcEmXDNnKvSLUwW/CQZIVq4wDwYDVR0RBAgwBocE\nfwAAATAKBggqhkjOPQQDAwNnADBkAjA8Ci9myzieoLoIy+7mUswVEjUG3wrEXtxA\nDRmt2PK9bTDo2m3aJ4nCQJtCWQRUlN0CMCMOsXL4NnfsSxaG5CwAVkDwLBUPv7Zy\nLfSh2oZ3Wn4FTxL0UfnJeFOz/CkDUtJI1A==\n-----END CERTIFICATE-----\n",
                "leafComponents": {
                    "claimsDigest": "d08d8764437d09b2d4d07d52293cddaf40f44a3ea2176a0528819a80002df9f6",
                    "commitEvidence": "ce:2.13:850a25da46643fa41392750b6ca03c7c7d117c27ae14e3322873de6322aa7cd3",
                    "writeSetDigest": "6637eddb8741ab54cc8a44725be67fd9be390e605f0537e5a278703860ace035",
                },
                "nodeId": "0db9a22e9301d1167a2a81596fa234642ad24bc742451a415b8d653af056795c",
                "proof": [
                    {
                        "left": "bcce25aa51854bd15257cfb0c81edc568a5a5fa3b81e7106c125649db93ff599"
                    },
                    {
                        "left": "cc82daa27e76b7525a1f37ed7379bb80f6aab99f2b36e2e06c750dd9393cd51b"
                    },
                    {
                        "left": "c53a15cbcc97e30ce748c0f44516ac3440e3e9cc19db0852f3aa3a3d5554dfae"
                    },
                ],
                "signature": "MGYCMQClZXVAFn+vflIIikwMz64YZGoH71DKnfMr3LXkQ0lhljSsvDrmtmi/oWwOsqy28PsCMQCMe4n9aXXK4R+vY0SIfRWSCCfaADD6teclFCkVNK4317ep+5ENM/5T/vDJf3V4IvI=",
            },
            "-----BEGIN CERTIFICATE-----\nMIIBvTCCAUOgAwIBAgIQGgBXsl9DwUaREaZ/qyKevDAKBggqhkjOPQQDAzAWMRQw\nEgYDVQQDDAtDQ0YgTmV0d29yazAeFw0yMzA0MjUxODE0MTlaFw0yMzA3MjQxODE0\nMThaMBYxFDASBgNVBAMMC0NDRiBOZXR3b3JrMHYwEAYHKoZIzj0CAQYFK4EEACID\nYgAER8wP5Jm3xNdLjOmVJllpAqhE+rSG8RUpLqpYpThB7MlggAZkhekDQz6G27ma\n6fqqJQH1rJciopwuDVRVGMIb6iSxrrQ1CuDzqGs5t2sbyRmylicldebkmk9FTY1P\n0vIlo1YwVDASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1UdDgQWBBTbkp6NwSZcM2cq\n9ItTBb8JBkhWrjAfBgNVHSMEGDAWgBTbkp6NwSZcM2cq9ItTBb8JBkhWrjAKBggq\nhkjOPQQDAwNoADBlAjAHTALS7/P02s1AeSWQHrVZdgz9F++NjFa0cea7TjLa1aPl\n/Hr20J6fjuFgK0GPn+ECMQDnsje32P4rKabgSgX4GPd5h97zuZp4o1QQJJ4hrjy4\nxQY4JS7/UrUrqpITDL082Vk=\n-----END CERTIFICATE-----\n",
            [
                {
                    "kind": "LedgerEntry",
                    "ledgerEntry": {
                        "collectionId": "subledger:0",
                        "contents": "Hello world",
                        "protocol": "LedgerEntryV1",
                        "secretKey": "Jde/VvaIfyrjQ/B19P+UJCBwmcrgN7sERStoyHnYO0M=",
                    },
                }
            ],
        ],
        [
            {
                "cert": "-----BEGIN CERTIFICATE-----\nMIIBxTCCAUygAwIBAgIRAMR89lUNeIghDUfpyHi3QzIwCgYIKoZIzj0EAwMwFjEU\nMBIGA1UEAwwLQ0NGIE5ldHdvcmswHhcNMjMwNDI1MTgxNDE5WhcNMjMwNzI0MTgx\nNDE4WjATMREwDwYDVQQDDAhDQ0YgTm9kZTB2MBAGByqGSM49AgEGBSuBBAAiA2IA\nBB1DiBUBr9/qapmvAIPm1o3o3LRViSOkfFVI4oPrw3SodLlousHrLz+HIe+BqHoj\n4nBjt0KAS2C0Av6Q+Xg5Po6GCu99GQSoSfajGqmjy3j3bwjsGJi5wHh1pNbPmMm/\nTqNhMF8wDAYDVR0TAQH/BAIwADAdBgNVHQ4EFgQUCPaDohOGjVgQ2Lb8Pmubg7Y5\nDJAwHwYDVR0jBBgwFoAU25KejcEmXDNnKvSLUwW/CQZIVq4wDwYDVR0RBAgwBocE\nfwAAATAKBggqhkjOPQQDAwNnADBkAjA8Ci9myzieoLoIy+7mUswVEjUG3wrEXtxA\nDRmt2PK9bTDo2m3aJ4nCQJtCWQRUlN0CMCMOsXL4NnfsSxaG5CwAVkDwLBUPv7Zy\nLfSh2oZ3Wn4FTxL0UfnJeFOz/CkDUtJI1A==\n-----END CERTIFICATE-----\n",
                "leafComponents": {
                    "claimsDigest": "d08d8764437d09b2d4d07d52293cddaf40f44a3ea2176a0528819a80002df9f6",
                    "commitEvidence": "ce:2.13:850a25da46643fa41392750b6ca03c7c7d117c27ae14e3322873de6322aa7cd3",
                    "writeSetDigest": "6637eddb8741ab54cc8a44725be67fd9be390e605f0537e5a278703860ace035",
                },
                "nodeId": "0db9a22e9301d1167a2a81596fa234642ad24bc742451a415b8d653af056795c",
                "proof": [
                    {
                        "left": "bcce25aa51854bd15257cfb0c81edc568a5a5fa3b81e7106c125649db93ff599"
                    },
                    {
                        "left": "cc82daa27e76b7525a1f37ed7379bb80f6aab99f2b36e2e06c750dd9393cd51b"
                    },
                    {
                        "left": "c53a15cbcc97e30ce748c0f44516ac3440e3e9cc19db0852f3aa3a3d5554dfae"
                    },
                ],
                "signature": "MGYCMQClZXVAFn+vflIIikwMz64YZGoH71DKnfMr3LXkQ0lhljSsvDrmtmi/oWwOsqy28PsCMQCMe4n9aXXK4R+vY0SIfRWSCCfaADD6teclFCkVNK4317ep+5ENM/5T/vDJf3V4IvI=",
            },
            "-----BEGIN CERTIFICATE-----\nMIIBvTCCAUOgAwIBAgIQGgBXsl9DwUaREaZ/qyKevDAKBggqhkjOPQQDAzAWMRQw\nEgYDVQQDDAtDQ0YgTmV0d29yazAeFw0yMzA0MjUxODE0MTlaFw0yMzA3MjQxODE0\nMThaMBYxFDASBgNVBAMMC0NDRiBOZXR3b3JrMHYwEAYHKoZIzj0CAQYFK4EEACID\nYgAER8wP5Jm3xNdLjOmVJllpAqhE+rSG8RUpLqpYpThB7MlggAZkhekDQz6G27ma\n6fqqJQH1rJciopwuDVRVGMIb6iSxrrQ1CuDzqGs5t2sbyRmylicldebkmk9FTY1P\n0vIlo1YwVDASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1UdDgQWBBTbkp6NwSZcM2cq\n9ItTBb8JBkhWrjAfBgNVHSMEGDAWgBTbkp6NwSZcM2cq9ItTBb8JBkhWrjAKBggq\nhkjOPQQDAwNoADBlAjAHTALS7/P02s1AeSWQHrVZdgz9F++NjFa0cea7TjLa1aPl\n/Hr20J6fjuFgK0GPn+ECMQDnsje32P4rKabgSgX4GPd5h97zuZp4o1QQJJ4hrjy4\nxQY4JS7/UrUrqpITDL082Vk=\n-----END CERTIFICATE-----\n",
            [
                {
                    "kind": "ClaimDigest",
                    "digest": {
                        "protocol": "LedgerEntryV1",
                        "value": "feb516ef1f903c64f1e388d9ee9fde11f64d1e2bc170248828c9eab9894f9ab9",
                    },
                }
            ],
        ],
    ],
)
def test_receipt_verification_with_valid_application_claims_returns_successfully(
    input_receipt, input_service_cert, input_claims
):
    # Check that verify_receipt does not throw any exception
    # with a valid receipt, service certificate, and application claims
    try:
        verify_receipt(
            input_receipt, input_service_cert, application_claims=input_claims
        )
    except Exception as e:
        pytest.fail(
            f"verify_receipt threw an exception with a valid receipt, service certificate, and application claims {e}"
        )


@pytest.mark.parametrize(
    "input_receipt,input_service_cert,input_claims",
    [
        [
            get_test_valid_receipt_1_dict(),
            get_test_valid_service_certificate_1(),
            get_test_application_claims_with_claim_digest_dict(),
        ],
        [
            get_test_valid_receipt_2_dict(),
            get_test_valid_service_certificate_2(),
            get_test_application_claims_with_ledger_entry_dict(),
        ],
    ],
)
def test_receipt_verification_with_invalid_application_claims_throws_exception(
    input_receipt, input_service_cert, input_claims
):
    # Check that verify_receipt throws ValueError
    with pytest.raises(
        ValueError,
    ):
        verify_receipt(
            input_receipt, input_service_cert, application_claims=input_claims
        )



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""Algorithm implementation for computing Azure Confidential Ledger application claims."""

from base64 import b64decode
from hashlib import sha256
import hmac
from typing import Any, Dict, List, cast

from azure.confidentialledger.receipt._claims_models import (
    ApplicationClaim,
    LedgerEntryClaim,
    ClaimDigest,
)

LEDGER_ENTRY_CLAIM_TYPE = "LedgerEntry"
DIGEST_CLAIM_TYPE = "ClaimDigest"
LEDGER_ENTRY_V1_CLAIM_PROTOCOL = "LedgerEntryV1"


def compute_claims_digest(application_claims: List[Dict[str, Any]]) -> str:
    """
    Compute the claims digest from a list of Azure Confidential Ledger application claims.

    :param application_claims: List of application claims to be verified against the receipt.
    :type application_claims: List[Dict[str, Any]]

    :return: The claims digest of the application claims.
    :rtype: str
    :raises ValueError: If the claims digest computation has failed.
    """

    # The logic is structured in three distinct steps:
    # 1. Claim dictionary validation
    # 2. Claim conversion to object
    # 3. Claims digest computation
    # For every step, we iterate over the list of application claims to process them one by one.
    # While it could be slightly inefficient to have three sets of iterations instead of a single one,
    # the main idea is to have a clean separation of concerns between the different steps
    # (e.g., not mix syntax/formatting errors with logical errors).
    # From a performance perspective, it is not expected to have a large number of application
    # claims in a single receipt (at the time of writing, we only support a single claim per receipt),
    # so the overhead of having three iterations should be negligible.

    # Validate application claims provided by the user
    _validate_application_claims(application_claims)

    # Convert application claims JSON objects to ApplicationClaim model
    application_claims_obj = []
    for claim_dict in application_claims:
        claim = ApplicationClaim.from_dict(claim_dict)
        application_claims_obj.append(claim)

    # Compute claims digest from application claims
    return _compute_claims_hexdigest(application_claims_obj)


def _validate_application_claims(application_claims: List[Dict[str, Any]]):
    """Validate the application claims in a write transaction receipt.

    :param list[dict[str, any]] application_claims: List of application claims to be verified against the receipt.
    """

    assert isinstance(application_claims, list)
    assert len(application_claims) > 0, "Application claims list cannot be empty"

    # Assert on each application claim object in the list
    for application_claim_object in application_claims:
        assert isinstance(application_claim_object, dict)

        # Assert on the kind of the claim
        assert "kind" in application_claim_object
        claim_kind = application_claim_object["kind"]
        assert isinstance(claim_kind, str)

        # Assert on the ledger entry claim
        if claim_kind == "LedgerEntry":
            ledger_entry_claim = application_claim_object.get("ledgerEntry")
            assert isinstance(ledger_entry_claim, dict)

            # Assert on the collection id
            assert "collectionId" in ledger_entry_claim
            assert isinstance(ledger_entry_claim["collectionId"], str)

            # Assert on the contents id
            assert "contents" in ledger_entry_claim
            assert isinstance(ledger_entry_claim["contents"], str)

            # Assert on the protocol
            assert "protocol" in ledger_entry_claim
            assert isinstance(ledger_entry_claim["protocol"], str)

            # Assert on the secret key
            assert "secretKey" in ledger_entry_claim
            assert isinstance(ledger_entry_claim["secretKey"], str)

        # Assert on the digest claim
        elif claim_kind == "ClaimDigest":
            assert "digest" in application_claim_object
            digest_claim = application_claim_object["digest"]
            assert isinstance(digest_claim, dict)

            # Assert on the digest value
            assert "value" in digest_claim
            assert isinstance(digest_claim["value"], str)

            # Assert on the protocol
            assert "protocol" in digest_claim
            assert isinstance(digest_claim["protocol"], str)

        else:
            assert False, f"Unknown claim kind: {claim_kind}"


def _compute_ledger_entry_v1_claim_digest(
    ledger_entry_claim: LedgerEntryClaim,
) -> bytes:
    """Compute the digest of a LedgerEntryV1 claim. It returns the digest in bytes.

    :param LedgerEntryClaim ledger_entry_claim: LedgerEntry claim to be digested.
    :return: The digest of the LedgerEntry claim.
    :rtype: bytes
    """

    # Decode the secret key
    secret_key = b64decode(ledger_entry_claim.secretKey, validate=True)

    # HMAC the collection ID with the secret key
    collection_id_digest = hmac.new(
        secret_key,
        ledger_entry_claim.collectionId.encode(),
        sha256,
    ).digest()

    # HMAC the ledger contents with the secret key
    contents_digest = hmac.new(
        secret_key,
        ledger_entry_claim.contents.encode(),
        sha256,
    ).digest()

    # Compute the SHA-256 of the concatenation of the collection ID and contents digests
    return sha256(collection_id_digest + contents_digest).digest()


def _compute_ledger_entry_claim_digest(ledger_entry_claim: LedgerEntryClaim) -> bytes:
    """Compute the digest of a LedgerEntry claim. It returns the digest in bytes.

    :param LedgerEntryClaim ledger_entry_claim: LedgerEntry claim to be digested.
    :return: The digest of the LedgerEntry claim.
    :rtype: bytes
    """

    claim_protocol = ledger_entry_claim.protocol

    # Compute the digest based on the specified protocol
    if claim_protocol == LEDGER_ENTRY_V1_CLAIM_PROTOCOL:
        # Compute the digest of the LedgerEntryV1 claim
        ledger_entry_digest = _compute_ledger_entry_v1_claim_digest(ledger_entry_claim)

    else:
        raise ValueError(f"Unsupported claim protocol: {claim_protocol}")

    # Compute the SHA-256 of the concatenation of the protocol and the ledger entry digest
    return sha256(claim_protocol.encode() + ledger_entry_digest).digest()


def _compute_claim_digest_from_object(claim_digest_object: ClaimDigest) -> bytes:
    # Compute the SHA-256 of the concatenation of the protocol and the digest value
    return sha256(
        claim_digest_object.protocol.encode() + bytes.fromhex(claim_digest_object.value)
    ).digest()


def _compute_claims_hexdigest(application_claims_list: List[ApplicationClaim]) -> str:
    """Compute the CCF claims digest from the provided list of application claims objects.
    It returns the hexdigest of the claims digest.

    :param list[ApplicationClaim] application_claims_list: List of application claims to be digested.
    :return: The hexdigest of the claims digest.
    :rtype: str
    """

    # Initialize the claims digest
    claims_digests_concatenation = b""

    # Iterate through all the application claims objects to compute their single digest.
    # We assume that the order of the application objects is valid
    # and the digests will be concatenated in the same order.
    for application_claim_object in application_claims_list:
        # Get the kind of the claim
        claim_kind = application_claim_object.kind

        if claim_kind == LEDGER_ENTRY_CLAIM_TYPE:
            # Compute the digest of the LedgerEntry claim
            claim_digest = _compute_ledger_entry_claim_digest(
                cast(LedgerEntryClaim, application_claim_object.ledgerEntry)
            )

        elif claim_kind == DIGEST_CLAIM_TYPE:
            # Compute the digest of the ClaimDigest claim
            claim_digest = _compute_claim_digest_from_object(
                cast(ClaimDigest, application_claim_object.digest)
            )

        else:
            raise ValueError(f"Unsupported claim kind: {claim_kind}")

        # Append the computed digest to the result
        claims_digests_concatenation += claim_digest

    # Prepend the size of application claims to the concatenation of the digests
    claims_digests_concatenation = (
        len(application_claims_list).to_bytes(length=4, byteorder="little")
        + claims_digests_concatenation
    )

    # Hash the concatenation of application claims and return the digest in hexadecimal form
    return sha256(claims_digests_concatenation).hexdigest()



#!/usr/bin/env python

# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
# pylint:disable=missing-docstring

from io import open
import os
import re
from setuptools import setup, find_packages

# example setup.py Feel free to copy the entire "azure-template" folder into a package folder named
# with "azure-<yourpackagename>". Ensure that the below arguments to setup() are updated to reflect
# your package.

PACKAGE_NAME = "azure-confidentialledger"
PACKAGE_PPRINT_NAME = "Confidential Ledger"

# a-b-c => a/b/c
PACKAGE_FOLDER_PATH = PACKAGE_NAME.replace("-", "/")
# a-b-c => a.b.c
NAMESPACE_NAME = PACKAGE_NAME.replace("-", ".")

# Version extraction inspired from 'requests'
with open(os.path.join(PACKAGE_FOLDER_PATH, "_version.py"), "r") as fd:
    VERSION = re.search(
        r'^VERSION\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

if not VERSION:
    raise RuntimeError("Cannot find version information")

with open("README.md", encoding="utf-8") as f:
    README = f.read()
with open("CHANGELOG.md", encoding="utf-8") as f:
    CHANGELOG = f.read()

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    include_package_data=True,
    description="Microsoft Azure {} Client Library for Python".format(
        PACKAGE_PPRINT_NAME
    ),
    # ensure that these are updated to reflect the package owners' information
    long_description=README + "\n\n" + CHANGELOG,
    long_description_content_type="text/markdown",
    license="MIT License",
    author="Microsoft Corporation",
    author_email="accledgerdevs@microsoft.com",
    url="https://github.com/Azure/azure-sdk-for-python",
    keywords="azure, azure sdk",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires=">=3.8",
    zip_safe=False,
    packages=find_packages(
        exclude=[
            "tests",
            # Exclude packages that will be covered by PEP420 or nspkg
            "azure",
        ]
    ),
    package_data={
        'pytyped': ['py.typed'],
    },
    install_requires=[
        "azure-core<2.0.0,>=1.28.0",
        "isodate<1.0.0,>=0.6.1",
        "cryptography>=2.1.4",
    ],
    project_urls={
        'Bug Reports': 'https://github.com/Azure/azure-sdk-for-python/issues',
        'Source': 'https://github.com/Azure/azure-sdk-for-python',
    }
)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------


"""Customize generated code here.

Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
import os
from typing import Any, List, Union

from azure.core.credentials_async import AsyncTokenCredential
from azure.core.pipeline import policies

from azure.confidentialledger.aio._client import (
    ConfidentialLedgerClient as GeneratedClient,
)

# Since we can't `await` in __init__, use the sync client for the Identity Service.


from azure.confidentialledger.certificate import ConfidentialLedgerCertificateClient
from azure.confidentialledger._patch import ConfidentialLedgerCertificateCredential

__all__: List[str] = [
    "ConfidentialLedgerClient",
]  # Add all objects you want publicly available to users at this package level


def patch_sdk():
    """Do not remove from this file.

    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """


class ConfidentialLedgerClient(GeneratedClient):
    """The ConfidentialLedgerClient writes and retrieves ledger entries against the Confidential
    Ledger service.

    :param endpoint: The Confidential Ledger URL, for example
     https://contoso.confidentialledger.azure.com.
    :type endpoint: str
    :param credential: A credential object for authenticating with the Confidential Ledger.
    :type credential: Union[
        ~azure.confidentialledger.ConfidentialLedgerCertificateCredential,
        ~azure.core.credentials_async.AsyncTokenCredential]
    :keyword ledger_certificate_path: The path to the Confidential Ledger's TLS certificate. If this
        file does not exist yet, the Confidential Ledger's TLS certificate will be fetched and saved
        to this file.
    :paramtype ledger_certificate_path: Union[bytes, str, os.PathLike]
    :keyword api_version: Api Version. Default value is "2022-05-13". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    """

    def __init__(
        self,
        endpoint: str,
        credential: Union[ConfidentialLedgerCertificateCredential, AsyncTokenCredential],
        *,
        ledger_certificate_path: Union[bytes, str, os.PathLike],
        **kwargs: Any,
    ) -> None:
        # Remove some kwargs first so that there aren't unexpected kwargs passed to
        # get_ledger_identity.

        if isinstance(credential, ConfidentialLedgerCertificateCredential):
            auth_policy = None
        else:
            credential_scopes = kwargs.pop("credential_scopes", ["https://confidential-ledger.azure.com/.default"])
            auth_policy = kwargs.pop(
                "authentication_policy",
                # Don't call AsyncBearerTokenCredentialPolicy here as an event loop may not be
                # running if a non-async authentication policy is passed.
                None,
            )

            if auth_policy is None:
                auth_policy = policies.AsyncBearerTokenCredentialPolicy(credential, *credential_scopes, **kwargs)
        if os.path.isfile(ledger_certificate_path) is False:
            # We'll need to fetch the TLS certificate.

            identity_service_client = ConfidentialLedgerCertificateClient(**kwargs)

            # Ledger URIs are of the form https://<ledger id>.confidential-ledger.azure.com.

            ledger_id = endpoint.replace("https://", "").split(".")[0]

            # We use the sync client here because async __init__ is not allowed.

            ledger_cert = identity_service_client.get_ledger_identity(ledger_id, **kwargs)

            with open(ledger_certificate_path, "w", encoding="utf-8") as outfile:
                outfile.write(ledger_cert["ledgerTlsCertificate"])
        # For ConfidentialLedgerCertificateCredential, pass the path to the certificate down to the
        # PipelineCLient.

        if isinstance(credential, ConfidentialLedgerCertificateCredential):
            # The async version of the client seems to expect a sequence of filenames.
            # azure/core/pipeline/transport/_aiohttp.py:163
            # > ssl_ctx.load_cert_chain(*cert)

            kwargs["connection_cert"] = kwargs.get("connection_cert", (credential.certificate_path,))
        # The auto-generated client has authentication disabled so we can customize authentication.
        # If the credential is the typical TokenCredential, then construct the authentication policy
        # the normal way.

        else:
            kwargs["authentication_policy"] = auth_policy
        # Customize the underlying client to use a self-signed TLS certificate.

        kwargs["connection_verify"] = kwargs.get("connection_verify", ledger_certificate_path)

        super().__init__(endpoint, **kwargs)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------

from ._receipt_verification import verify_receipt
from ._claims_digest_computation import compute_claims_digest

__all__ = ["verify_receipt", "compute_claims_digest"]
